#include <vector>
#include "searchtimer_thread.h"
#include "epgsearchcfg.h"
#include "epgsearchcats.h"
#include "epgsearchtools.h"
#include "changrp.h"
#include "switchtimer.h"
#include <vdr/tools.h>
#include <vdr/plugin.h>
#include "switchtimer_thread.h"
#include "services.h"
#include "conflictcheck.h"
#include "uservars.h"
#include "svdrpclient.h"

// priority for background thread
#define SEARCHTIMER_NICE 19
// delay before first thread activity
#define SEARCHTIMER_WAIT 20

extern int updateForced;

cSearchTimerThread *cSearchTimerThread::m_Instance = NULL;

cSearchTimerThread::cSearchTimerThread(cPluginEpgsearch* thePlugin)
: cThread("EPGSearch: searchtimer")
{
    m_plugin = thePlugin;
    m_Active = false;
    m_lastUpdate = time(NULL);
}

cSearchTimerThread::~cSearchTimerThread() {
    if (m_Active) 
	Stop();
}

void cSearchTimerThread::Init(cPluginEpgsearch* thePlugin, bool activatePermanently) 
{
    if (activatePermanently)
    {
	EPGSearchConfig.useSearchTimers = 1;
	thePlugin->SetupStore("UseSearchTimers",  EPGSearchConfig.useSearchTimers);
    }
    if (!EPGSearchConfig.useSearchTimers)
	return;    
    if (m_Instance == NULL) {
	m_Instance = new cSearchTimerThread(thePlugin);
	m_Instance->Start();
    }
}

void cSearchTimerThread::Exit(void) {
    if (m_Instance != NULL) {
	m_Instance->Stop();
	DELETENULL(m_Instance);
    }
}

void cSearchTimerThread::Stop(void) {
    m_Active = false;
    Cancel(6);
}


cTimer *cSearchTimerThread::GetTimer(cSearchExt *searchExt, const cEvent *pEvent, bool& bTimesMatchExactly)
{
    cChannel *channel = Channels.GetByChannelID(pEvent->ChannelID(), true);
    if (!channel)
	return NULL;

    struct tm tm_r;

    bool UseVPS = searchExt->useVPS && pEvent->Vps() && Setup.UseVps;
    time_t eStart = pEvent->StartTime();
    time_t eStop = pEvent->EndTime();

    int testVpsStart = 0;
    int testVpsStop = 0;
    time_t testVpsDay = 0;
    if (UseVPS)
    {
	eStart = pEvent->Vps();
	eStop = eStart + pEvent->Duration();
	cTimer VpsTestTimer(pEvent);
	testVpsStart = cTimer::TimeToInt(VpsTestTimer.Start());
	testVpsStop = cTimer::TimeToInt(VpsTestTimer.Stop());
	testVpsDay = VpsTestTimer.Day();
    }

    tm *tmStartEv = localtime_r(&eStart, &tm_r);
    
    for (cTimer *ti = Timers.First(); ti; ti = Timers.Next(ti)) 
    {
	if (ti->Channel() != channel)
	    continue;

	if (ti->WeekDays()) // ignore serial timers
	    continue;

	if (UseVPS && ti->HasFlags(tfVps))
	{
	    if (testVpsDay != ti->Day()) continue;
	    int timerVpsStart = cTimer::TimeToInt(ti->Start());
	    int timerVpsStop = cTimer::TimeToInt(ti->Stop());

	    if (abs(testVpsStart - timerVpsStart) > 10 * 60) continue;
	    if (abs(testVpsStop - timerVpsStop) > 10 * 60) continue;

	    bTimesMatchExactly = (testVpsStart == timerVpsStart && testVpsStop == timerVpsStop);
	    return ti;
	}
	else
	{
	    time_t tStart = ti->StartTime() + searchExt->MarginStart * 60;
	    time_t tStop = ti->StopTime() - searchExt->MarginStop * 60;
	    tm *tmStartTi = localtime_r(&tStart, &tm_r);
	    if (tmStartEv->tm_mday != tmStartTi->tm_mday)
		continue;
	    
	    bTimesMatchExactly = (tStart == eStart && tStop == eStop);
	
	    if (abs(tStart - eStart) < 10 * 60 && abs(tStop - eStop) < 10 * 60) // accept a difference of max 10 min.
		return ti;
	}
    }
    return NULL;
}

bool cSearchTimerThread::TimerWasModified(cTimer* t)
{
    if (!t) return false;
    if (t->HasFlags(tfVps)) return false; // if timer uses VPS we ignore user changes

    char* start = GetAuxValue(t, "Start");
    char* stop = GetAuxValue(t, "Stop");
    bool bMod = false;
    if (start || stop)
    {
	time_t StartTime = time_t(atol(start));
	time_t StopTime = time_t(atol(stop));
	if (t->StartTime() != StartTime || t->StopTime() != StopTime)
	    bMod = true;
    }
    if (start) free(start);
    if (stop) free(stop);
    return bMod;
}

void cSearchTimerThread::Action(void) 
{
    if (EPGSearchConfig.useExternalSVDRP && !cSVDRPClient::SVDRPSendCmd)
    {
	LogFile.eSysLog("ERROR - SVDRPSend script not specified or does not exist (use -f option)");
	return;
    }
    SetPriority(SEARCHTIMER_NICE);

    m_Active = true;
    // let VDR do its startup
    sleepSec(SEARCHTIMER_WAIT);
    time_t nextUpdate = time(NULL);
    while (m_Active) 
    {
	time_t now = time(NULL);
	bool needUpdate = NeedUpdate();
	if (now >= nextUpdate || needUpdate)
	{
	    if (Timers.BeingEdited()) 		
	    {
		sleepSec(1);
		continue;
	    }
	    LogFile.iSysLog("search timer update started");

	    UserVars.ResetCache(); // reset internal cache of user vars
	    cTimerObjList* pOutdatedTimers = NULL;

            // for thread safeness we work with a copy of the current searches,
            // because SVDRP would not work if the main thread would be locked 
	    cSearchExts* localSearchExts = SearchExts.Clone(); 

	    cSearchExt *searchExt = localSearchExts->First();
	    while (searchExt) 
	    {
		if (!searchExt->useAsSearchTimer)
		{
		    searchExt = localSearchExts->Next(searchExt);
		    continue;
		}
		pOutdatedTimers = searchExt->GetTimerList(pOutdatedTimers);

		cSearchResults* pSearchResults = searchExt->Run(-1, true);
		if (!pSearchResults)
		{
		    searchExt = localSearchExts->Next(searchExt);
		    continue;
		}
		pSearchResults->SortBy(CompareEventTime);

		if (searchExt->pauseOnNrRecordings > 0)
		    searchExt->CheckExistingRecordings(pSearchResults);

		for (cSearchResult* pResultObj = pSearchResults->First(); 
		     pResultObj; 
		     pResultObj = pSearchResults->Next(pResultObj))
		{
			const cEvent* pEvent = pResultObj->event;
			if (!pEvent)
			    continue;
			    
			cChannel *channel = Channels.GetByChannelID(pEvent->ChannelID(), true);
			if (!channel)
			    continue;
			
			int index = 0;
			cTimer *timer = new cTimer(pEvent);

			// create the file 
			char* file = NULL;
			if ((file = searchExt->BuildFile(pEvent)) != NULL)
			{
			    while(strstr(file, "!^pipe^!")) file = strreplace(file, "!^pipe^!", "|"); // revert the translation of '|' in BuildFile 
			    timer->SetFile(file);
			    free(file);
			}
			int Priority = searchExt->Priority;
			int Lifetime = searchExt->Lifetime;
			
			// search for an already existing timer
			bool bTimesMatchExactly = false;
			cTimer *t = GetTimer(searchExt, pEvent, bTimesMatchExactly);

			char* Summary = NULL;
			if (t) 
			{ // already exists
			    pOutdatedTimers->DelTimer(t);

			    if (!t->HasFlags(tfActive))
			    { // do not update inactive timers
				LogFile.Log(2,"timer for '%s~%s' (%s - %s, channel %d) not active - won't be touched", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent));
				delete timer;
				continue;
			    }
			    
			    if (!pResultObj->needsTimer && !t->Recording()) // not needed
			    {
 				LogFile.Log(1,"delete timer for '%s~%s' (%s - %s, channel %d)", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent));
				RemoveTimer(t, pEvent);
				delete timer;
				continue;
			    }
			    if (TimerWasModified(t)) // don't touch timer modified by user 
			    {
 				LogFile.Log(2,"timer for '%s~%s' (%s - %s, channel %d) modified by user - won't be touched", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent));
				delete timer;
				continue;
			    }
			    int triggerID = TriggeredFromSearchTimerID(t);
			    if (triggerID > -1 && triggerID != searchExt->ID)
			    {
 				LogFile.Log(2,"timer for '%s~%s' (%s - %s, channel %d) already created by search id %d - won't be touched", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent), triggerID);
				delete timer;
				continue;
			    }

			    char* pFile = NULL; // File is prepared for svdrp, so prepare t->File for comparision too
			    asprintf(&pFile, "%s", t->File());
			    pFile = strreplace(pFile, ':', '|');
			    pFile = strreplace(pFile, " ~", "~");
			    pFile = strreplace(pFile, "~ ", "~");
			    
			    Summary =  SummaryExtended(searchExt, t, pEvent);
			    
			    if (bTimesMatchExactly && strcmp(pFile, timer->File()) == 0
				&& (t->Aux() != NULL && strcmp(t->Aux(), Summary) == 0)
				)
			    { // dir, title, episode name and summary have not changed
				if (Summary) free(Summary);
				delete timer;
				free(pFile);
				continue;
			    }
			    else
			    {
				index = t->Index()+1;
				Priority = t->Priority();
				Lifetime = t->Lifetime();
			    }
			    free(pFile);
			}			       
			else
			{
			    if (!pResultObj->needsTimer)
			    {
				delete timer;
				continue;
			    }
			}
		
			if (index == 0 && searchExt->action == searchTimerActionAnnounceOnly) // only announce if there is no timer for the event
			{
			    char* Message = NULL;
			    asprintf(&Message, "%s: %s - %s %s", pEvent->Title(), CHANNELNAME(channel), GETDATESTRING(pEvent), GETTIMESTRING(pEvent));
			    bool addTimer = false;
			    if (SendMsg(Message, true,7) == kRed)			
			    {
				if (SendMsg(tr("Create a timer for this event?"), true, 5) == kOk)			
				    addTimer = true;
			    }
			    free(Message);


			    if (!addTimer)
			    {
				sleepSec(5);
				if (Summary) free(Summary);
				delete timer;
				continue;
			    }
			}

			if (searchExt->action == searchTimerActionSwitchOnly) // add to switch list
			{
			    time_t now = time(NULL);
			    if (now < pEvent->StartTime())
			    {
				if (!SwitchTimers.InSwitchList(pEvent))
				{			
				    cMutexLock SwitchTimersLock(&SwitchTimers);	
				    SwitchTimers.Add(new cSwitchTimer(pEvent, searchExt->switchMinsBefore));
				    SwitchTimers.Save();
				    cSwitchTimerThread::Init(); 
				}
			    }
			    if (Summary) free(Summary);
			    delete timer;
			    continue;
			}
			
			AddModTimer(timer, index, searchExt, pEvent, Priority, Lifetime, Summary);
			if (Summary) free(Summary);
			delete timer;
			if (index == 0)
			    LogFile.Log(1,"added timer for '%s~%s' (%s - %s); search timer: '%s'", pEvent->Title(), pEvent->ShortText()?pEvent->ShortText():"", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), searchExt->search);
			else
			    LogFile.Log(1,"modified timer %d for '%s~%s' (%s - %s); search timer: '%s'", index, pEvent->Title(), pEvent->ShortText()?pEvent->ShortText():"", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), searchExt->search);
		}
		searchExt = localSearchExts->Next(searchExt);
	    }
	    if (localSearchExts) delete localSearchExts;

	    if (pOutdatedTimers)
	    {
		if (pOutdatedTimers->Count() > 0)
		{
		    LogFile.Log(1,"removing outdated timers");
		    for(cTimerObj *tObj = pOutdatedTimers->First(); tObj; tObj = pOutdatedTimers->Next(tObj))
		    {
			cTimer* t = tObj->timer;
			// timer could have been deleted meanwhile, so check if its still there
			bool found = false;
                        for(cTimer* checkT = Timers.First(); checkT; checkT = Timers.Next(checkT)) 
			    if (checkT == t)
			    {
				found = true;
				break;
			    }
			if (!found) continue;

			if (TimerWasModified(t)) continue;
			if (!t->Event()) continue; // if there is no event, we keep the timer, since EPG could have been cleared
			time_t now = time(NULL);
			if (abs(t->StopTime() - now) < 120) continue; // if this timer has just finished, let VDR do the cleanup
			LogFile.Log(1,"delete timer for '%s' (%s, channel %s)", t->File(), DAYDATETIME(t->StartTime()), CHANNELNAME(t->Channel()));
			RemoveTimer(t);
		    }
		    LogFile.Log(1,"removing outdated timers - done");
		}
		delete pOutdatedTimers;
	    }

	    LogFile.iSysLog("search timer update finished");

	    // check for conflicts
	    if (EPGSearchConfig.checkTimerConflictsAfterUpdate)
	    {
		LogFile.iSysLog("check for timer conflicts");
		cConflictCheck conflictCheck;   
		conflictCheck.Check(); 

		if (conflictCheck.relevantConflicts > 0)
		{
		    cList<cConflictCheckTime>* failedList = conflictCheck.GetFailed(); // get the first relevant conflict
		    cConflictCheckTime* ct = failedList->First();
		    for(; ct; ct = failedList->Next(ct))
			if (ct->ignore) 
			    continue; 
			else 
			    break;

		    if (EPGSearchConfig.sendMailOnConflicts)
		    {
			cMailConflictNotifier mailNotifier;
			mailNotifier.SendConflictNotifications(conflictCheck);
		    }

		    char* msgfmt = NULL;
		    asprintf(&msgfmt, tr("%d timer conflict(s)! First at %s. Show them?"), conflictCheck.relevantConflicts,
			    *DateTime(ct->evaltime));
		    if (SendMsg(msgfmt, true,7) == kOk)			
		    {
			m_plugin->showConflicts = true;
			cRemote::CallPlugin("epgsearch");	
		    }
		    free(msgfmt);
		}
		
		LogFile.iSysLog("check for timer conflicts - done");	    
	    }

	    // delete expired recordings
	    CheckExpiredRecs();

	    mailNotifier.SendUpdateNotifications();

	    if (updateForced == 2)
		SendMsg(tr("Search timer update done!"));

	    // reset service call flag
	    updateForced = 0;

	    m_lastUpdate = time(NULL);
	    nextUpdate = long(m_lastUpdate/60)*60 + (EPGSearchConfig.UpdateIntervall * 60);
	}
	sleepSec(2); // to avoid high system load if time%30==0 
	while (m_Active && !NeedUpdate() && time(NULL)%30 != 0) // sync heart beat to a multiple of 5secs
	    sleepSec(1);
    };
    LogFile.iSysLog("Leaving search timer thread");
}

bool cSearchTimerThread::NeedUpdate()
{
    return  (m_lastUpdate <= LastModifiedTime(AddDirectory(CONFIGDIR, ".epgsearchupdate")) || updateForced>0);
}

int cSearchTimerThread::GetCountTimers(const cEvent* event, bool compareTitle, bool compareSubtitle, bool compareSummary, unsigned long catvaluesAvoidRepeat, cTimer** first)
{
    if (!event) return 0;

    *first = NULL;
    int count = 0;
    for (cTimer *ti = Timers.First(); ti; ti = Timers.Next(ti)) 
    {
	if (!ti->HasFlags(tfActive))
	    continue;

	const cEvent* t_event = ti->Event();
	if (!t_event)
	    t_event = GetEvent(ti);
	if (!t_event)
	    continue;

	if (EventsMatch(event, t_event, compareTitle, compareSubtitle, compareSummary, catvaluesAvoidRepeat))
	{
	    if (!*first) *first = ti;
	    else
		if ((*first)->StartTime() > ti->StartTime())
		    *first = ti;	    
	    count++;
	}
    }
    return count;
}

char* cSearchTimerThread::SummaryExtended(cSearchExt* searchExt, cTimer* Timer, const cEvent* pEvent)
{
    char* addSummaryHeader = NULL;
    const char* title = pEvent->Title();
    const char* subtitle = pEvent->ShortText();
    char sep = '\n';

    bool UseVPS = searchExt->useVPS && pEvent->Vps() && Setup.UseVps;
    time_t eStart;
    if (!UseVPS)
	eStart = pEvent->StartTime();
    else
	eStart = pEvent->Vps();
    time_t eStop;
    if (!UseVPS)
	eStop = pEvent->EndTime();
    else
	eStop = pEvent->Vps() + pEvent->Duration();
    time_t start = eStart - (UseVPS?0:(searchExt->MarginStart * 60));
    time_t stop  = eStop + (UseVPS?0:(searchExt->MarginStop * 60));

    sep = '|';

    asprintf(&addSummaryHeader, "%s%c%s%s%s", 
	     title?title:"", sep,
	     subtitle?tr("Subtitle"):"", subtitle?": ":"", subtitle?subtitle:"");

    char* addSummaryFooter = NULL;

    asprintf(&addSummaryFooter, "<Channel>%d - %s</Channel><Search timer>%s</Search timer><Start>%ld</Start><Stop>%ld</Stop><S-ID>%d</S-ID>", 
	     Timer->Channel()->Number(), CHANNELNAME(Timer->Channel()),
	     searchExt->search, 
	     start, 
	     stop, 
	     searchExt->ID);

    const char* aux = Timer->Aux();
    // remove epgsearch entries
    char* tmpaux = NULL;
    if (!isempty(aux))
    {
	tmpaux = strdup(aux);
	char* begin = strstr(aux, "<epgsearch>");
	char* end = strstr(aux, "</epgsearch>");
	if (begin && end)
	{
	    if (begin == aux) strcpy(tmpaux, ""); else strn0cpy(tmpaux, aux, begin-aux+1);
	    strcat(tmpaux, end + strlen("</epgsearch>"));
	}
    }

    char* tmpSummary = NULL;
    asprintf(&tmpSummary, "<epgsearch>%s</epgsearch>%s", addSummaryFooter, tmpaux?tmpaux:"");
    free(addSummaryHeader);
    free(addSummaryFooter);
    if (tmpaux) free(tmpaux);
    return tmpSummary;
}

void cSearchTimerThread::AddModTimer(cTimer* Timer, int index, cSearchExt* searchExt, const cEvent* pEvent, int Prio, int Lifetime, char* Summary) 
{
    char *cmdbuf = NULL;

    static char bufStart[25];
    static char bufEnd[25];

    struct tm tm_r;    time_t eStart = pEvent->StartTime();
    time_t eStop = pEvent->EndTime();
    time_t start = eStart - (searchExt->MarginStart * 60);
    time_t stop  = eStop + (searchExt->MarginStop * 60);
    int Flags = Timer->Flags();
    if (searchExt->useVPS && pEvent->Vps() && Setup.UseVps)
    {
	start = pEvent->Vps();
	stop = start + pEvent->Duration();
    }
    else
	Flags = 1; // don't use VPS, if not set in this search

    strftime(bufStart, sizeof(bufStart), "%H%M", localtime_r(&start, &tm_r));
    strftime(bufEnd, sizeof(bufEnd), "%H%M", localtime_r(&stop, &tm_r));

    // add additonal info
    char* tmpSummary = NULL;
    if (Summary)
    {
	tmpSummary = strdup(Summary);
	strreplace(tmpSummary, '\n', '|');
    }
    else 
	tmpSummary = SummaryExtended(searchExt, Timer, pEvent);

    if (index==0)
	asprintf(&cmdbuf, "NEWT %d:%d:%s:%s:%s:%d:%d:%s:%s", 
		 Flags,
		 Timer->Channel()->Number(),
		 *Timer->PrintDay(start, Timer->WeekDays()),
		 bufStart,
		 bufEnd,
		 Prio, 
		 Lifetime, 
		 Timer->File(),
		 tmpSummary?tmpSummary:"");
    else
	asprintf(&cmdbuf, "MODT %d %d:%d:%s:%s:%s:%d:%d:%s:%s", 
		 index,
		 Flags,
		 Timer->Channel()->Number(),
		 *Timer->PrintDay(start, Timer->WeekDays()),
		 bufStart,
		 bufEnd,
		 Prio, 
		 Lifetime, 
		 Timer->File(),
		 tmpSummary?tmpSummary:"");

    SendViaSVDRP(cmdbuf);

    if (EPGSearchConfig.sendMailOnSearchtimers)
    {
	if (index==0) // new
	    mailNotifier.AddNewTimerNotification(pEvent->EventID(), pEvent->ChannelID()); 
	else
	    mailNotifier.AddModTimerNotification(pEvent->EventID(), pEvent->ChannelID()); 
    }
    free(cmdbuf);
    if (tmpSummary) free(tmpSummary);
}

void cSearchTimerThread::RemoveTimer(cTimer* t, const cEvent* e)
{
    if (!t) return;
    if (EPGSearchConfig.sendMailOnSearchtimers)
	mailNotifier.AddRemoveTimerNotification(t,e); 
    DelTimer(t->Index()+1);
}

void cSearchTimerThread::DelRecording(int index)
{
    char *cmdbuf = NULL;   // first LSTR required 
    asprintf(&cmdbuf, "DELR %d", index);    
    LogFile.Log(2, "delete recording %d", index);
    SendViaSVDRP(cmdbuf);
	      
    free(cmdbuf); 
}

void cSearchTimerThread::CheckExpiredRecs()
{
    LogFile.Log(1, "check for expired recordings started");
    cThreadLock RecordingsLock(&Recordings);
    cList<cRecordingObj> DelRecordings;
    for (cRecording *recording = Recordings.First(); recording; recording = Recordings.Next(recording)) 
    {
	LogFile.Log(3, "check recording %s from %s for expiration", recording->Name(), DAYDATETIME(recording->start));
	if (recording->start == 0)
	{
	    LogFile.Log(2, "oops, recording %s has no start time, skipped", recording->Name());
	    continue;
	}
	if (recording->IsEdited()) continue;

	if (!recording->Info()) continue;
	char* searchID = GetAuxValue(recording, "S-ID");	
	char* searchName = GetAuxValue(recording, "Search timer");	
	if (!searchName)
	    searchName = GetAuxValue(recording, tr("Search timer"));	

	if (!searchID || !searchName)
	{
	    if (searchID) free(searchID);
	    if (searchName) free(searchName);
	    continue;
	}
	cSearchExt* search = SearchExts.GetSearchFromID(atoi(searchID));
	if (!search || strcmp(search->search, searchName) != 0) 
	{
	    if (searchID) free(searchID);
	    if (searchName) free(searchName);
	    continue;
	}
	LogFile.Log(3, "recording triggered from search timer %s", search->search);
	if (search->delAfterDays == 0) continue;
	time_t now = time(NULL);
	int daysBetween = int(double((now - recording->start)) / (60*60*24));
	if (daysBetween  >= search->delAfterDays)
	    DelRecordings.Add(new cRecordingObj(recording, search));
	else
	    LogFile.Log(3, "recording will expire in %d days, search timer %s", search->delAfterDays - daysBetween, search->search);
    }    
    for (cRecordingObj *recordingObj = DelRecordings.First(); recordingObj; recordingObj = DelRecordings.Next(recordingObj)) 
    {
	cRecording* recording = recordingObj->recording;
	cSearchExt* search = recordingObj->search;
	if (search->recordingsKeep > 0 && search->recordingsKeep >= search->GetCountRecordings())
	{
	    LogFile.Log(1, "recording '%s' from %s expired, but will be kept, search timer %s", recording->Name(), DAYDATETIME(recording->start), recordingObj->search->search);
	    continue;
	}
	LogFile.Log(1, "delete expired recording '%s' from %s, search timer %s", recording->Name(), DAYDATETIME(recording->start), recordingObj->search->search);
	cRecordControl *rc = cRecordControls::GetRecordControl(recording->FileName());
	if (!rc) 
	{
	    if (!recording->Delete())
		LogFile.Log(1, "error deleting recording!");
	    else
		::Recordings.DelByName(recording->FileName());
	}
	else
	    LogFile.Log(1, "recording already in use by a timer!");
    }
    LogFile.Log(1, "check for expired recordings finished");
}
